### test_scale_update_asm.s: Contains wrapper functions which are
### used to init registers to known "garbage" values before calling a
### student-written function. This helps force out "used a random
### register" problems that will have variable behavior on different
### machines.

.text
.global CALL_scale_from_ports
.type	CALL_scale_from_ports, @function
CALL_scale_from_ports:       
        ## %rdi is a pointer argument
        .cfi_startproc

        ## 8 bytes on stack at the start for return address
        ## 6 * 8 bytes pushed
        ## need an addtional 8 bytes to align the stack for a call
        subq $56, %rsp

        ## changes ot the rsp require annotation to allow debuggers to
        ## know where the previous stack frame starts. after the above
        ## extension of the stack, the previous stack fraom is 64
        ## bytes above rsp
        .cfi_def_cfa_offset 64

        ## save 6 callee save registers except the stack pointer
        movq %rbp,  0(%rsp)
        movq %rbx,  8(%rsp)
        movq %r12, 16(%rsp)
        movq %r13, 24(%rsp)
        movq %r14, 32(%rsp)
        movq %r15, 40(%rsp)

        ## initialize all registers to ensure they have "garbage"
        ## values in them
        movabsq $0xBADBADBADBADBADD, %rax
        movabsq $0xBADBADBADBADBADD, %rbx
        movabsq $0xBADBADBADBADBADD, %rcx
        movabsq $0xBADBADBADBADBADD, %rdx
        movabsq $0xBADBADBADBADBADD, %rsi
        ## movabsq $0xBADBADBADBADBADD, %rdi  # pointer to tod struct to fill
        ## movabsq $0xBADBADBADBADBADD, %rsp
        movabsq $0xBADBADBADBADBADD, %rbp
        movabsq $0xBADBADBADBADBADD, %r8
        movabsq $0xBADBADBADBADBADD, %r9
        movabsq $0xBADBADBADBADBADD, %r10
        movabsq $0xBADBADBADBADBADD, %r11
        movabsq $0xBADBADBADBADBADD, %r12
        movabsq $0xBADBADBADBADBADD, %r13
        movabsq $0xBADBADBADBADBADD, %r14
        movabsq $0xBADBADBADBADBADD, %r15

        ## call target function
        call scale_from_ports

        ## store callee regs to check them later
        call get_callee_regs
        
        ## restore 6 callee registers
        movq  0(%rsp), %rbp
        movq  8(%rsp), %rbx
        movq 16(%rsp), %r12
        movq 24(%rsp), %r13
        movq 32(%rsp), %r14
        movq 40(%rsp), %r15

        addq $56, %rsp
	.cfi_def_cfa_offset 8   # annotate another rsp change for debugger
        
        ## rax has the return value from the function
        ret                     
.cfi_endproc
.size CALL_scale_from_ports,.-CALL_scale_from_ports

.global CALL_scale_display_special
.type	CALL_scale_display_special, @function
CALL_scale_display_special:
        ## %rdi is packed struct arg
        ## %rsi is pointer arg
        .cfi_startproc
        subq $56, %rsp          # see first function for details
        .cfi_def_cfa_offset 64

        ## save 6 callee save registers except the stack pointer
        movq %rbp,  0(%rsp)
        movq %rbx,  8(%rsp)
        movq %r12, 16(%rsp)
        movq %r13, 24(%rsp)
        movq %r14, 32(%rsp)
        movq %r15, 40(%rsp)

        ## initialize all registers to ensure they have "garbage"
        ## values in them
        movabsq $0xBADBADBADBADBADD, %rax
        movabsq $0xBADBADBADBADBADD, %rbx
        movabsq $0xBADBADBADBADBADD, %rcx
        ## movabsq $0xBADBADBADBADBADD, %rdx  # pointer to display variable
        ## movabsq $0xBADBADBADBADBADD, %rsi  # later fields of tod_t struct
        ## movabsq $0xBADBADBADBADBADD, %rdi  # first fields of tod_t struct
        ## movabsq $0xBADBADBADBADBADD, %rsp
        movabsq $0xBADBADBADBADBADD, %rbp
        movabsq $0xBADBADBADBADBADD, %r8
        movabsq $0xBADBADBADBADBADD, %r9
        movabsq $0xBADBADBADBADBADD, %r10
        movabsq $0xBADBADBADBADBADD, %r11
        movabsq $0xBADBADBADBADBADD, %r12
        movabsq $0xBADBADBADBADBADD, %r13
        movabsq $0xBADBADBADBADBADD, %r14
        movabsq $0xBADBADBADBADBADD, %r15

        ## call the target function
        call scale_display_special

        ## store callee regs to check them later
        call get_callee_regs

        ## restore 6 callee registers
        movq  0(%rsp), %rbp
        movq  8(%rsp), %rbx
        movq 16(%rsp), %r12
        movq 24(%rsp), %r13
        movq 32(%rsp), %r14
        movq 40(%rsp), %r15

        addq $56, %rsp
	.cfi_def_cfa_offset 8   # annotate another rsp change for debugger

        ## rax has the return value from the function
        ret                     
.cfi_endproc
.size CALL_scale_display_special,.-CALL_scale_display_special

.global CALL_scale_display_weight
.type	CALL_scale_display_weight, @function
CALL_scale_display_weight:
        ## %rdi is packed struct arg
        ## %rsi is pointer arg
        .cfi_startproc
        subq $56, %rsp          # see first function for details
        .cfi_def_cfa_offset 64

        ## save 6 callee save registers except the stack pointer
        movq %rbp,  0(%rsp)
        movq %rbx,  8(%rsp)
        movq %r12, 16(%rsp)
        movq %r13, 24(%rsp)
        movq %r14, 32(%rsp)
        movq %r15, 40(%rsp)

        ## initialize all registers to ensure they have "garbage"
        ## values in them
        movabsq $0xBADBADBADBADBADD, %rax
        movabsq $0xBADBADBADBADBADD, %rbx
        movabsq $0xBADBADBADBADBADD, %rcx
        ## movabsq $0xBADBADBADBADBADD, %rdx  # pointer to display variable
        ## movabsq $0xBADBADBADBADBADD, %rsi  # later fields of tod_t struct
        ## movabsq $0xBADBADBADBADBADD, %rdi  # first fields of tod_t struct
        ## movabsq $0xBADBADBADBADBADD, %rsp
        movabsq $0xBADBADBADBADBADD, %rbp
        movabsq $0xBADBADBADBADBADD, %r8
        movabsq $0xBADBADBADBADBADD, %r9
        movabsq $0xBADBADBADBADBADD, %r10
        movabsq $0xBADBADBADBADBADD, %r11
        movabsq $0xBADBADBADBADBADD, %r12
        movabsq $0xBADBADBADBADBADD, %r13
        movabsq $0xBADBADBADBADBADD, %r14
        movabsq $0xBADBADBADBADBADD, %r15

        ## call the target function
        call scale_display_weight

        ## store callee regs to check them later
        call get_callee_regs

        ## restore 6 callee registers
        movq  0(%rsp), %rbp
        movq  8(%rsp), %rbx
        movq 16(%rsp), %r12
        movq 24(%rsp), %r13
        movq 32(%rsp), %r14
        movq 40(%rsp), %r15

        addq $56, %rsp
	.cfi_def_cfa_offset 8   # annotate another rsp change for debugger

        ## rax has the return value from the function
        ret                     
.cfi_endproc
.size   CALL_scale_display_weight,.-CALL_scale_display_weight


.global CALL_scale_update
.type	CALL_scale_update, @function
CALL_scale_update:     
        ## no arguments
        .cfi_startproc
        subq $56, %rsp          # see first function for details
        .cfi_def_cfa_offset 64

        ## save 6 callee save registers except the stack pointer
        movq %rbp,  0(%rsp)
        movq %rbx,  8(%rsp)
        movq %r12, 16(%rsp)
        movq %r13, 24(%rsp)
        movq %r14, 32(%rsp)
        movq %r15, 40(%rsp)

        ## initialize all registers to ensure they have "garbage"
        ## values in them
        movabsq $0xBADBADBADBADBADD, %rax
        movabsq $0xBADBADBADBADBADD, %rbx
        movabsq $0xBADBADBADBADBADD, %rcx
        movabsq $0xBADBADBADBADBADD, %rdx
        movabsq $0xBADBADBADBADBADD, %rsi
        movabsq $0xBADBADBADBADBADD, %rdi
        ## movabsq $0xBADBADBADBADBADD, %rsp
        movabsq $0xBADBADBADBADBADD, %rbp
        movabsq $0xBADBADBADBADBADD, %r8
        movabsq $0xBADBADBADBADBADD, %r9
        movabsq $0xBADBADBADBADBADD, %r10
        movabsq $0xBADBADBADBADBADD, %r11
        movabsq $0xBADBADBADBADBADD, %r12
        movabsq $0xBADBADBADBADBADD, %r13
        movabsq $0xBADBADBADBADBADD, %r14
        movabsq $0xBADBADBADBADBADD, %r15

        ## call the target function
        call scale_update

        ## store callee regs to check them later
        call get_callee_regs

        ## restore 6 callee registers
        movq  0(%rsp), %rbp
        movq  8(%rsp), %rbx
        movq 16(%rsp), %r12
        movq 24(%rsp), %r13
        movq 32(%rsp), %r14
        movq 40(%rsp), %r15

        addq $56, %rsp
	.cfi_def_cfa_offset 8   # annotate another rsp change for debugger

        ## rax has the return value from the function
        ret                     
.cfi_endproc
.size CALL_scale_update, .-CALL_scale_update

### store the callee register values in an array to later check that
### they have not changed
get_callee_regs:
        .cfi_startproc
        leaq callee_reg_vals(%rip), %rdi
        movq %rbp,  0(%rdi) 
        movq %rbx,  8(%rdi) 
        movq %r12, 16(%rdi) 
        movq %r13, 24(%rdi) 
        movq %r14, 32(%rdi) 
        movq %r15, 40(%rdi) 
        ret
	.cfi_endproc
